//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  coord.hpp
//! \brief Coordinates CCoord, numeric parameters CParam.


//! A coordinate, consisting of a name and a unit.

class CCoord {
 public:
    string name;
    string unit;

    CCoord() {}
    //! Creates instance with given name and unit.
    CCoord( const string _name, const string _unit ) :
        name(_name), unit(_unit) {}
    //! Creates instance. Given string will be parsed into name and unit.
    CCoord( const string in );

    void ask_and_set( const string& quest );

    friend std::ostream& operator<< (std::ostream &s, CCoord C);
    string str_std() const;
    string str_compact( int maxlen=0 ) const;
    string str_ps() const;
    string to_a01() const;
    string load_a01(string);

    //! Has this instance nonvoid contents?
    bool defined() { return name!=""; }
    //! Are two instances equal?
    bool operator== (const CCoord& Co2) const {
        return (name==Co2.name) && (unit==Co2.unit); }
    //! Returns negation of operator==.
    bool operator!= (const CCoord& Co2) const {
        return !(*this==Co2); }
};


//! A file-wide parameter, consisting of a coordinate and a value.

class CParam {
 public:
    CCoord Co;
    double val;
    double dval; //!< Error of val. Still unused.

    //! Creates instance with fully given contents.
    CParam( CCoord _Co, double _val=0, double _dval=0 )
        : Co(_Co), val(_val), dval(_dval) {}
    //! Creates instance with fully given, partly unparsed contents.
    CParam( string _name, string _unit="", double _val=0, double _dval=0 ) :
        Co( CCoord(_name, _unit) ), val(_val), dval(_dval) {}
};