//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  fregistry.cpp
//! \brief Registration of CFunc's.

#include "defs.hpp"


#include "func.hpp"
#include "fregistry.hpp"


//! Prints a list of registered operators, for online help.

void SFuncRegistry::display_operators() const
{
    cout << "Operators by precedence:\n";
    for( auto f: FList ){
        if( !f->precedence )
            continue;
        cout << "  " << f->explanation << "\n";
    }
}


//! Prints a list of registered CFunc's, for online help.

void SFuncRegistry::display_functions() const
{
    cout << "\nBuilt-in functions:\n";
    for( auto f: FList ){
        if( f->precedence )
            continue;
        cout << "  " << f->tag << f->explanation << "\n";
    }
}


//! Registers a typed function implementation.
//! The function name must have been previously registered using register_fct_meta.

void SFuncRegistry::register_fct_template(
    const CFuncMetadata& _m, const char* _outtype, const char* _intypes, funcPtr _f )
{
    auto pos = FMap.find(_m.tag);
    CFunc* F;
    if( pos == FMap.end() ) {
        F = new CFunc( _m );
        FMap.insert( std::make_pair( _m.tag, F ) );
        FList.push_back( F );
    } else
        F = pos->second;
    F->tyfuList.push_back( new CTypedFunc( _outtype, _intypes, _f ) );
}
