//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  loop.cpp
//! \brief Implements iterator over file selection.

#include "defs.hpp"
#include "olf.hpp"
#include "mem.hpp"
#include "loop.hpp"

FileIterator::FileIterator( const vector<int>& _Sel, bool tolerate_empty )
{
    Sel = _Sel;
    for( int k: Sel )
        if( k<0 || k>=SMem::instance()->mem_size() )
            throw "invalid entry " + S(k) + " in file selection";
    if ( !tolerate_empty && !size() )
        throw S("empty file selection");
    reset();
}

int FileIterator::k() const
{
    if( iV<0 || iV>=Sel.size() )
        throw "BUG: FileIterator::k() called from invalid state";
    return Sel[iV];
}

COlo* FileIterator::current()
{
    if( iV<0 )
        throw "BUG: FileIterator::current() called from invalid state";
    if( iV>=Sel.size() )
        return nullptr;
    int k=Sel[iV];
    if( k<0 )
        throw S("BUG: k<0 in file selection");
    if( k>=SMem::instance()->mem_size() )
        throw "FileSelection["+S(iV)+"]="+S(k)+" exceeds online memory size "+S(SMem::instance()->mem_size());
    return SMem::instance()->mem_get(k);
}

COld* FileIterator::currentD()
{
    COlo *f = current();
    if( !f )
        return nullptr;
    if( COld *ret = dynamic_cast<COld*>(f) )
        return ret;
    throw S("BUG: non-D file found by D iterator");
}

COlc* FileIterator::currentC()
{
    COlo *f = current();
    if( !f )
        return nullptr;
    if( COlc *ret = dynamic_cast<COlc*>(f) )
        return ret;
    throw S("BUG: non-C file found by C iterator");
}
