//***************************************************************************//
//* libtrivia: various little routines for C++                              *//
//* integrate: wrapper for GSL integration routines                         *//
//* (C) Joachim Wuttke 2012-                                                *//
//* http://apps.jcns.fz-juelich.de                                          *//
//***************************************************************************//

#include <gsl/gsl_integration.h>
#include "integrate.hpp"

//! Compute integral of func() from low to high, using a GSL routine.

double triv::integrate( double (*func_) (double, void*), void *data_,
                        double low, double hig, int mode,
                        double epsabs, double epsrel )
{
    gsl_function F;
    F.function = func_;
    F.params = data_;
#define nwork 20000
    gsl_integration_workspace *work = gsl_integration_workspace_alloc( nwork );
    double val, err;
    size_t neval;

    if        ( isinf(low) && isinf(hig) ) {
        gsl_integration_qagi( &F, epsabs, epsrel, nwork, work, &val, &err );
    } else if ( isinf(low) ) {
        gsl_integration_qagil( &F, hig, epsabs, epsrel, nwork, work, &val, &err );
    } else if ( isinf(hig) ) {
        gsl_integration_qagiu( &F, low, epsabs, epsrel, nwork, work, &val, &err );
    } else if ( mode==0 ) {
        // QNG non-adaptive Gauss-Kronrod integration
        gsl_integration_qng( &F, low, hig, epsabs, epsrel,
                             &val, &err, &neval );
    } else if ( mode>0 && mode<7 ) {
        int rule;
        switch( mode ){
        case 1: rule = GSL_INTEG_GAUSS61; break;
        case 2: rule = GSL_INTEG_GAUSS51; break;
        case 3: rule = GSL_INTEG_GAUSS41; break;
        case 4: rule = GSL_INTEG_GAUSS31; break;
        case 5: rule = GSL_INTEG_GAUSS21; break;
        case 6: rule = GSL_INTEG_GAUSS15; break;
        }
        // QAG adaptive integration
        gsl_integration_qag( &F, low, hig, epsabs, epsrel, nwork, rule, work, &val, &err );
    } else if ( mode==7 ) {
        // QAGS adaptive integration with singularities
        gsl_integration_qags( &F, low, hig, epsabs, epsrel, nwork, work, &val, &err );
    } else {
        throw "invalid mode in mystd::Integrate";
    }
    gsl_integration_workspace_free( work );
    return val;
}
