//***************************************************************************//
//* libtrivia: various little routines for C++                              *//
//* string_ops: string operations                                           *//
//* (C) Joachim Wuttke 2001, 2012-                                          *//
//* http://apps.jcns.fz-juelich.de                                          *//
//***************************************************************************//

#include <cstring>

#include <iostream>


#include "string_ops.hpp"

using namespace std;

//! Extract floating-point numbers from string.

bool triv::str2vec( string inp, vector<double> *V, size_t nmax, bool force )
// ------ TODO: convert to void, throwing errors -------
{

    // cout << "DEBUG mystd [" << inp << "]\n";
    V->clear();

    while(inp.size() && isspace(inp[0]))
        inp.erase(inp.begin());
    // while(inp.size() && strchr(" \t.\n",inp[inp.size()-1]))
    //    inp.erase(inp.end()-1);

    // cout << "DEBUG  core [" << inp << "]\n";
    if (inp==string("")) {
        // cout << "DEBUG  empty\n";
        return true;
    }
    // cout << "DEBUG  nonempty\n";

    string::size_type ji=0, jf;
    double val;
    char dummy;
    while (ji<inp.size()) {
        if(inp[ji]=='#')
            break;
        if ((jf=inp.find_first_of(", \t\n", ji)) == string::npos)
            jf = inp.size();
        // cout << " look until " << jf << "\n";
        if (sscanf(inp.substr(ji,jf-ji).c_str(), "%lg%c", &val, &dummy)==1) {
            V->push_back(val);
        } else if ( force ) {
            cout << "! str2vec: invalid value ["
                 << inp.substr(ji,jf-ji) << "] at pos " << ji
                 << " in [" << inp << "]\n";
            return false;
        }
        if (nmax && V->size()>=nmax)
            break; // do not decode the tail
        // cout << " found "<<ji<<":"<<jf<<" value "<<val<<"\n";
        ji = inp.find_first_not_of(", \t\n", jf);
        // cout << " continue at " << ji << "\n";
    }

    return true;
}

//! Strip leading whitespace (or whatever characters given by sepSet).

string triv::strip(
    string const& str, ///< the original string
    char const* sepSet ///< C string with characters to be dropped
)
{
    string::size_type const first = str.find_first_not_of(sepSet);
    return ( first==std::string::npos )
        ? std::string()
        : str.substr(first, str.find_last_not_of(sepSet)-first+1);
}

//! Split string at first non-leading whitespace, ignoring whitespace in  `..`.

//! one word (*out1) is extracted; leading " \t" are retained.
//! in the tail (*out2) leading " \t" are deleted.
//! "\n" starts always a new word.
//! leading "`" makes *out1 go to next "`" or to end of line.

void triv::string_extract_word( const string& in, string *out1, string *out2 )
{
    string::size_type j0=0, j1, j2, jf=in.size();

    // handle special case "`"
    if (in[0]=='`') {
        j1 = in.find_first_of('`', 1);
        if (j1==string::npos) {
            *out1 = in.substr(1);
            *out2 = "";
        } else {
            *out1 = in.substr(1, j1-1);
            j2 = in.find_first_not_of(" \t", j1+1);
            if (j2!=string::npos)
                *out2 = in.substr(j2);
            else
                *out2 = "";
        }
        return;
    }

    while (j0<jf && strchr(" \t",in[j0])) j0++;
    j1=j0+1;
    while (j1<jf && !strchr(" \t\n",in[j1])) j1++;
    j2=j1;
    while (j2<jf && strchr(" \t",in[j2])) j2++;
    *out1 = in.substr(0,j1);
    if (j2<jf)
        *out2 = in.substr(j2);
    else
        *out2 = string("");
}

//! Split string at first linebreak.

void triv::string_extract_line( const string& in, string *out1, string *out2 )
{
    string line;
    string::size_type i = in.find('\n');
    if (i==string::npos) {
        *out1 = in;
        *out2 = string("");
    } else {
        *out1 = in.substr(0,i);
        *out2 = in.substr(i+1);
    }
}

//! Split string at given delimiters, return vector of tokens.

vector<string> triv::split( const string& in, const string& delimiters )
{
    vector<string> out;
    // skip delimiters at beginning:
    string::size_type lastPos = in.find_first_not_of( delimiters, 0 );
    // find first non-delimiter:
    string::size_type pos = in.find_first_of( delimiters, lastPos );
    while ( pos!=string::npos || lastPos!=string::npos ) {
        // found a token, add it to the vector:
        out.push_back( in.substr(lastPos, pos-lastPos) );
        // skip delimiters:
        lastPos = in.find_first_not_of( delimiters, pos );
        // find next non-delimiter:
        pos = in.find_first_of( delimiters, lastPos );
    }
    return out;
}
